﻿declare const cw: cwWebSDK.IServices;

declare namespace cwWebSDK {
    /* Web SDK services */
    interface IServices {
        version: string;
        onReady: (callback: Function) => void;
        initialize: (options: IInitializationOptions) => void;
        PaymentForm: (target: HTMLElement, options: cwWebSDK.IPaymentFormOptions) => cwWebSDK.IPaymentForm;
        PaymentConfirmation: (form: HTMLFormElement, options: cwWebSDK.IPaymentConfirmationOptions) => void;
        UserAgreement: (target: HTMLElement, options: IUserAgreementOptions) => void;
    }

    /* Web SDK initialization options */

    export type CardPaymentProvider = "" | "cyberSourceWithTokenEx" | "cyberSource" | "vestaWithTokenEx" | "payonWithPCIProxy" | "payon";
    export type BankAccountPaymentProvider = "" | "sepa" | "payon";

    interface IInitializationOptions {
        apiURL?: string;
        locale?: string;
        enableCyberSource?: boolean;
        enablePayPal?: boolean;
        bankAccountPaymentProvider?: BankAccountPaymentProvider;
        cardPaymentProvider?: CardPaymentProvider;
    }

    /* User agreement options */

    export type UserAgreementMode = "INITIAL" | "REPEATED";

    interface IUserAgreementOptions {
        programCode: string;
        mode: UserAgreementMode;
        financialInstitutionName: string;
        fileName?: string;
        onBack?: () => void;
        onAccept?: (userAgreementReference: string) => void;
        onLoad?: () => void;
        onError?: (responseDescription: string, response: unknown) => void;
    }

    /* Payment form options */

    export type PaymentProviderMode = "live" | "test";
    export type PaymentOptionCode =
        | "MSTRCRD"
        | "VISA"
        | "VISADBIT"
        | "MSTRO"
        | "AMEX"
        | "CRTBANCAIR"
        | "PAYPAL"
        | "GIROPAY"
        | "IDEAL"
        | "BNKACCT"
        | "PAYU"
        | "PAYUBLK"
        | "PAYUTWST"
        | "PAYUINST"
        | "DISCOVER"
        | "JCB"
        | "VISAELTRN";

    interface IPaymentFormOptions
        extends Partial<IPaymentsOsOptions>,
            Partial<IPayPalOptions>,
            Partial<IPayonOptions>,
            Partial<ICyberSourceOptions>,
            Partial<ICyberSourceWithTokenExOptions>,
            Partial<IVestaWithTokenExOptions>,
            Partial<IPayonWithPCIProxyOptions>,
            Partial<ISepaOptions> {
        onBeforeSubmit?: (...args: unknown[]) => void;
        paymentInfoCallback?: (paymentData: PaymentInfo) => void;
        onError?: (errorMessage: string, error: Error) => void;
    }

    interface IPaymentForm {
        dispose: () => void;
    }

    interface ICardPaymentInfo {
        paymentBrand?: PaymentOptionCode;
        expiryDate?: string;
        cardHolder?: string;
    }

    interface IBankAccountPaymentInfo {
        iban?: string;
        accountHolder?: string;
        bankName?: string;
    }

    interface ICssClasses {
        [key: string]: string;
    }
    
    type PaymentInfo = ICardPaymentInfo | IBankAccountPaymentInfo;
    type Error = IPayonError | IWebApiError;

    /* Payment confirmation options */

    interface IPaymentConfirmationOptions {
        paymentProviderMode?: PaymentProviderMode;
        sessionIdentifier?: string;
    }

    /* PAY.ON payment form options */

    export type PayonPaymentBrand = "MASTER" | "VISA" | "VISADEBIT" | "MAESTRO" | "AMEX" | "CARTEBANCAIRE" | "JCB" | "PAYPAL" | "GIROPAY" | "IDEAL" | "DIRECTDEBIT_SEPA";

    interface IPayonOptions {
        authorizationToken: string;
        paymentOptionCodes: PaymentOptionCode[];
        paymentProviderMode: PaymentProviderMode;

        targetName?: string;

        locale?: string;

        showPlaceholders?: boolean;
        placeholdersColor?: string;

        isStorable?: boolean;
        isStorableCaption?: string;
        submitButtonTitle?: string;

        showLabels?: boolean;
        showCVVHint?: boolean;
        requireCvv?: boolean;

        callbackUrl?: string;
        confirmationUrl?: string;

        iframeCssClasses?: ICssClasses;
        iframeStyles?: IPayonStyles;

        onFormLoaded?: () => void;
        onError?: (message: string, error: IPayonError) => void;
        paymentInfoCallback?: (data: ICardPaymentInfo) => void;
        onBeforeSubmit?: (data: ICardPaymentInfo, submit: () => void) => void;
    }

    interface IPayonStyles {
        [key: string]: {
            [key: string]: unknown;
        };
    }

    interface IPayonError {
        message?: string;
    }

    /* PaymentsOs payment form options */

    interface IPaymentsOsOptions {
        authorizationToken: string;
        locale?: string;
    }

    /* PayPal payment form options */

    interface IPayPalOptions {
        authorizationToken: string;
        paymentProviderMode: PaymentProviderMode;
        trackingID?: string;
        flowIdentifier?: string;
        locale?: string;
        paypalButtonStyle?: IPayPalButtonStyle;
        onFormLoaded?: () => void;
        onBeforeSubmit?: () => void;
        onError?: (message: string, error: IPayPalError) => void;
        onComplete?: (data: IPayPalCompletePaymentData) => void;
    }

    interface IPayPalError {
        message: string;
    }

    interface IPayPalCompletePaymentData {
        preventRedirect: () => void;
        isRedirectPrevented: () => boolean;
        getPaymentProviderData: () => any;
    }

    interface IPayPalButtonStyle {
        color?: "" | "gold" | "blue" | "silver" | "white" | "black";
        shape?: "" | "pill" | "rect";
        size?: "" | "small" | "medium" | "large" | "responsive";
        height?: "" | "small" | "medium" | "large";
        label?: "" | "checkout" | "credit" | "pay" | "buynow" | "paypal" | "installment";
        tagline?: boolean;
        layout?: "" | "vertical" | "horizontal";
    }

    /* Web API payment form options */

    interface IWebApiPaymentFormOptions {
        authorizationToken: string;

        locale?: string;

        iframeCss?: string;
        iframeCssClasses?: ICssClasses;
        iframeCssUrls?: string[];

        showLabels?: boolean;

        submitButtonTitle?: string;

        paymentInfoCallback?: (data: PaymentInfo) => void;
        onBeforeSubmit?: (data: PaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
        onError?: (message: string, error: IWebApiError) => void;
    }

    interface IWebApiError {
        reason?: string;
        value?: string;
    }

    interface ICyberSourceOptions extends IWebApiPaymentFormOptions {
        paymentOptionCodes: PaymentOptionCode[];
        paymentProviderMode: PaymentProviderMode;

        isStorable?: boolean;
        isStorableCaption?: string;

        showPlaceholders?: boolean;

        paymentInfoCallback?: (data: ICardPaymentInfo) => void;
        onBeforeSubmit?: (data: ICardPaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
    }

    interface ICyberSourceWithTokenExOptions extends IWebApiPaymentFormOptions {
        paymentOptionCodes: PaymentOptionCode[];
        paymentProviderMode: PaymentProviderMode;

        isStorable?: boolean;
        isStorableCaption?: string;

        showPlaceholders?: boolean;

        paymentInfoCallback?: (data: ICardPaymentInfo) => void;
        onBeforeSubmit?: (data: ICardPaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
    }

    interface IVestaWithTokenExOptions extends IWebApiPaymentFormOptions {
        paymentOptionCodes: PaymentOptionCode[];
        paymentProviderMode: PaymentProviderMode;

        isStorable?: boolean;
        isStorableCaption?: string;

        showPlaceholders?: boolean;

        paymentInfoCallback?: (data: ICardPaymentInfo) => void;
        onBeforeSubmit?: (data: ICardPaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
    }

    interface IPayonWithPCIProxyOptions extends IWebApiPaymentFormOptions {
        paymentOptionCodes: PaymentOptionCode[];

        paymentInfoCallback?: (data: ICardPaymentInfo) => void;
        onBeforeSubmit?: (data: ICardPaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
    }

    interface ISepaOptions extends IWebApiPaymentFormOptions {
        showPlaceholders?: boolean;

        paymentInfoCallback?: (data: IBankAccountPaymentInfo) => void;
        onBeforeSubmit?: (data: IBankAccountPaymentInfo, submit: () => void, cancelSubmit: () => void) => void;
    }
}